;------------------------------------------------------------------------
;- Written by: Bazz
;-    Mode 7 Scaling Tutorial
;---> teaches its powers.
;------------------------------------------------------------------------

;== Include MemoryMap, HeaderInfo, and interrupt Vector table ==
.INCLUDE "header.inc"

;== Include Library Routines ==
.INCLUDE "InitSNES.asm"
.INCLUDE "LoadGraphics.asm"
.INCLUDE "Joypad.asm"

;== EQUates ==
.ENUM $00
sx			dw
sy			dw
scrollx		dw
scrolly		dw
.ENDE

;==========================================
; Main Code
;==========================================
.BANK 0 SLOT 0
.ORG 0
.SECTION "MainCode"

Start:
    InitSNES
    rep #$10
    sep #$20
    
;-- Load map data to vram --
    stz $2115	; increment vram address after write to $2118 (low byte)
    ldx #$0000	; vram addr $0000
    stx $2116
    ldx #$1800	; Write to $2118
    stx $4300
    ldx #Map
    stx $4302
    lda #:Map
    sta $4304
    ldy #$4000	; # of bytes
    sty $4305
    
    lda #$01	; Enable DMA channel 0
    sta $420B
    
;-- Load tile data to vram --
    lda #$80	; increment vram address after write to $2119 (high byte)
    sta $2115	
    ldx #$0000	; vram addr $0000
    stx $2116
    ldx #$1900	; Write to $2119
    stx $4300
    ldx #Tiles
    stx $4302
    lda #:Tiles
    sta $4304
    ldy #$1f80	; # of bytes
    sty $4305
    
    lda #$01
    sta $420B

    LoadPalette BG_Palette, 0, 128

    jsr SetupVideo
    
    lda #$0f
    sta $2100
    
    lda #$81
    sta $4200       ; Enable NMI
    stz $4016

Infinity:
	WAI
	jsr Joypad
_done:
    JMP Infinity

;============================================================================
VBlank:
	rep #$10
	sep #$20

    lda $4210       ; Clear NMI flag
    
    lda sx
    sta $211b
    lda sx+1
    sta $211b
    
    lda sy
    sta $211e
    lda sy+1
    sta $211e
    
    lda scrollx
    sta $210d
    sta $211f
    lda scrollx+1
    sta $210d
    sta $211f
    
    lda scrolly
    sta $210e
    sta $2120
    lda scrolly+1
    sta $210e
    sta $2120
    
    
    
    jsr UpdatePad
    RTI
;============================================================================

;============================================================================
; SetupVideo -- Sets up the video mode and tile-related registers
;----------------------------------------------------------------------------
; In: None
;----------------------------------------------------------------------------
; Out: None
;----------------------------------------------------------------------------
SetupVideo:
    lda #$07
    sta $2105           ; Set Video mode 7, 8x8 tiles

    lda #$00            ; Set BG1's Tile Map offset to $0000 (Word address)
    sta $2107           ; And the Tile Map size to 32x32

    stz $210B           ; Set BG1's Character VRAM offset to $0000 (word address)
    
    lda #$00
    sta $211a
    
    rep #$20
    lda #1<<8
    sta sx
    sta sy
    sep #$20
    sta $211b           ; mode 7 matrix parameter A register (low)
	xba					; switch low & high bytes
	sta $211b           ; Mode 7 matrix parameter A register (high)
	xba					; switch
	stz $211c           ; Mode 7 matrix parameter B register (low)
	stz $211c           ; Mode 7 matrix parameter B register (high)
	stz $211d           ; Mode 7 matrix parameter C register (low)
	stz $211d           ; Mode 7 matrix parameter C register (high)
	sta $211e           ; Mode 7 matrix parameter D register (low)
	xba
	sta $211e           ; Mode 7 matrix parameter D register (high)

	stz $211f           ; Mode 7 center position X register (low)
	stz $211f           ; Mode 7 center position X register (high)
	stz $2120           ; Mode 7 center position Y register (low)
	stz $2120           ; Mode 7 center position Y register (high)
	
	stz scrollx
	stz scrollx+1
	stz $210d
	stz $210d
	
	stz scrolly
	stz scrolly+1
	stz $210e
	stz $210e

    lda #$01            ; Enable BG1
    sta $212C

    rts

.ENDS
;============================================================================
.SECTION "CharacterData"
Map:
	.incbin "gfx/mariokart.mp7"

Tiles:
    .incbin "gfx/mariokart.pc7"

BG_Palette:
    .incbin "gfx/mariokart.clr"

.ENDS

